/* -*- c-basic-offset: 3; mode: c++ -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 * 
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

#include <math.h>
#include <pandore.h>
using namespace pandore;

#include "evaluation.h"

inline Float
WeightedHarmonicMean( Float e1, Float e2 , Float alpha ) {
   if (e1 == 1.0 && e2 == 1.0) {
      return 1.0f;
   } else {
      return (Float)(1.0 - (1.0 / ( (alpha / (1.0 - e2)) +  ((1.0 - alpha) / (1.0 - e1)))));
   }
}

inline EvaluationPerformance
ComputeIndicators( const EvaluationResult & evaluationResult,
		   int proposition1, int proposition2, int proposition3,
		   int proposition4, int proposition5 ) {
   int propositions[5] = {
      proposition1,
      proposition2,
      proposition3,
      proposition4,
      proposition5
   };
   
   EvaluationPerformance evaluationPerformance;

   for (int i = 0; i < 5; i++) {
	 switch (propositions[i]) {
	    case 1:
	       evaluationPerformance.indicators[i] = 0;
	       break;
	    case 2:
	       evaluationPerformance.indicators[i] = WeightedHarmonicMean(evaluationResult.errors[i * 2], evaluationResult.errors[i * 2 + 1], 0.5f);
	       break;
	    case 3:
	       evaluationPerformance.indicators[i] = WeightedHarmonicMean(evaluationResult.errors[i * 2], evaluationResult.errors[i * 2 + 1], 0.8f);
	       break;
	    case 4:
	       evaluationPerformance.indicators[i] = WeightedHarmonicMean(evaluationResult.errors[i * 2], evaluationResult.errors[i * 2 + 1], 0.2f);
	       break;
	    case 5:
	       evaluationPerformance.indicators[i] = WeightedHarmonicMean(evaluationResult.errors[i * 2], evaluationResult.errors[i * 2 + 1], 1.0f);
	       break;
	    case 6:
	       evaluationPerformance.indicators[i] = WeightedHarmonicMean(evaluationResult.errors[i * 2], evaluationResult.errors[i * 2 + 1], 0.0f);
	       break;
	    case 7:
	       if (evaluationResult.errors[i * 2] == 0) {
		  evaluationPerformance.indicators[i] = evaluationResult.errors[i * 2 + 1];
	       } else {
		  evaluationPerformance.indicators[i] = 1.0;
	       }
	       break;
	    case 8:
	       if (evaluationResult.errors[1 * 2 + 1] == 0) {
		  evaluationPerformance.indicators[i] = evaluationResult.errors[i * 2];
	       } else {
		  evaluationPerformance.indicators[i] = 1.0;
	       }
	       break;
	    default:
	       std::cerr << "Error : Invalid paramater #" << (i + 1) << "="
			 << propositions[i] << ". Should be in range [1-8]."
			 << std::endl;
	       return evaluationPerformance;
	 }
   }
   return evaluationPerformance;
}
