/* -*- c-basic-offset: 3; mode: c++ -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are


make * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 * 
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

#if (defined _MSC_VER) && (!defined snprintf)
#define snprintf _snprintf
#endif

inline float
RankOrderCentroid( int priority, int numberOfIndicators ) {
   float weight = 0.0f;
   for (int j = priority; j <= numberOfIndicators; j++) {
      weight += 1.0f / j;
   }
   return weight / numberOfIndicators;
}

inline void
RankAlgorithmsInIndicators( EvaluationPerformance *performanceMatrix, EvaluationPerformance *rankedPerformances, int nbOfAlgorithms, int indicator ) {
   float *values = new float[nbOfAlgorithms];
   int size = 0;
   
   for (int a = 0; a < nbOfAlgorithms; a++) {
      bool exist = false;
      for (int b = 0; b < size; b++) {
	 if (performanceMatrix[a].indicators[indicator] == values[b] ) {
	    exist = true;
	    break;
	 }
      }
      if (!exist) {
	 values[size++] = performanceMatrix[a].indicators[indicator];
      }
   }
   
   for (int a = 0; a < nbOfAlgorithms; a++) {
      int nb = 1;
      for (int b = 0; b < size; b++) {
	 if (performanceMatrix[a].indicators[indicator] > values[b]) {
	    nb++;
	 }
      }
      rankedPerformances[a].indicators[indicator] = (float)nb;
   }
}

inline Errc
MultiIndicatorRanking( EvaluationPerformance *performanceMatrix, Long *ranks,
		       int nbOfAlgorithms, int nbOfIndicators, int priority[] ) {
   std::string type;

   EvaluationPerformance *rankedPerformances = new EvaluationPerformance[nbOfAlgorithms];
   
   for (int i = 0; i < nbOfIndicators; i++) {
      RankAlgorithmsInIndicators(performanceMatrix, rankedPerformances, nbOfAlgorithms, i);
   }

   float *averageRanks = new float[nbOfAlgorithms];
   for (int a = 0; a < nbOfAlgorithms; a++) {
      averageRanks[a] = 0;
      for (int i = 0; i < nbOfIndicators; i++) {
	 averageRanks[a] += rankedPerformances[a].indicators[i] * RankOrderCentroid(priority[i], nbOfIndicators);
      }
   }

   for (int a = 0; a < nbOfAlgorithms; a++) {
      int nb = 1;
      for (int b = 0; b < nbOfAlgorithms; b++) {
	 if (averageRanks[a] > averageRanks[b]) {
	    nb++;
	 }
      }
      ranks[a] = nb;
   }

   return SUCCESS;
}
