/* -*- c-basic-offset: 3; mode: c++ -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 * 
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

#include <iomanip>
#include <cstring>

#include <pandore.h>
using namespace pandore;

#ifdef MAIN
#undef MAIN
#include "./evaluation/log.h"
#include "./evaluation/computeindicators.h"
#include "./evaluation/multiindicatorranking.h"
#define MAIN
#else
#include "./evaluation/log.h"
#include "./evaluation/computeindicators.h"
#include "./evaluation/multiindicatorranking.h"
#endif

Errc
ReadCollection( const Collection & cols,  const std::string &errorName, int nbOfIndicators, EvaluationResult &evaluationResult ) {
   char tmp1[256];
   char tmp2[256];

   for (int i = 0; i < nbOfIndicators; i++) {
      snprintf(tmp1, sizeof(tmp1), "%s.%d", errorName.c_str(), 2 * i + 1);
      tmp1[sizeof(tmp1) - 1] = 0;
      std::string type1 = cols.GetType(tmp1);
      snprintf(tmp2, sizeof(tmp2), "%s.%d", errorName.c_str(), 2 * i + 2);
      tmp1[sizeof(tmp2) - 1] = 0;
      std::string type2 = cols.GetType(tmp2);
      
      if (type1 == "Float" && type2 == "Float") {
	 evaluationResult.errors[i * 2] = cols.GETVALUE(tmp1, Float);
	 evaluationResult.errors[i * 2 + 1] = cols.GETVALUE(tmp2, Float);
      } else {
	 std::cerr << "Error : Invalid collection contents" << std::endl;
	 return FAILURE;
      }
   }
   return SUCCESS;
}

void
displayIndicators( const EvaluationPerformance *performances, int nbOfAlgorithms, int nbOfIndicators ) {
   LOG(INFO) << std::endl << "    FINAL RESULTS" << std::endl;  
   LOG(INFO) << "+-----------+-------------+-------------+-------------+-------------+-------------+" << std::endl;
   LOG(INFO) << "| Algorithm | indicator 1 | indicator 2 | indicator 3 | indicator 4 | indicator 5 |" << std::endl;
   LOG(INFO) << "+-----------+-------------+-------------+-------------+-------------+-------------+" << std::endl;
   
   for (int element = 0; element < nbOfAlgorithms; element++) {
      LOG(INFO) << "|" << std::setw(10) << element + 1 << " |" ;
      for (int indicator = 0; indicator < nbOfIndicators; indicator++) {
	 LOG(INFO) << std::fixed << std::setw(12) << std::setprecision(3) << performances[element].indicators[indicator] << " |";
      }
      LOG(INFO) <<std::endl;
   }
   LOG(INFO) << "+-----------+-------------+------------+-------------+--------------+-------------+" << std::endl << std::endl;
}

void
displayRanks( const Long * ranks, int nbOfAlgorithms ) {
   LOG(INFO) << "+-------+-----------+" << std::endl;
   LOG(INFO) << "| Ranks | Algorithm |" << std::endl;
   LOG(INFO) << "+-------+-----------+" << std::endl;
   for (int r = 0; r < (int)nbOfAlgorithms; r++) {
      LOG(INFO) << "| " << std::setw(5) << (r + 1) << " | " << std::setw(9) << ranks[r] << " | " << std::endl;
   }
   LOG(INFO) << "+-------+-----------------+" << std::endl;
}

void
buildOutputResult1( const EvaluationPerformance * performances, Collection & cold1, int nbOfAlgorithms, int nbOfIndicators ) {
   const std::string nameOut = "Rank";
   Float **indicators = new Float*[nbOfIndicators];
   for (int i = 0; i < nbOfIndicators; i++) {
      indicators[i] = new Float[nbOfAlgorithms];
      for (int j = 0; j < nbOfAlgorithms; j++) {
	 indicators[i][j] =  performances[j].indicators[i];
      }
   }
   char tmp1[256];
   for (int i = 0; i < nbOfIndicators; i++) {
      snprintf(tmp1, sizeof(tmp1), "Indicator.%d", i + 1);
      tmp1[sizeof(tmp1) - 1] = 0;
      cold1.SETARRAY(tmp1, Float, indicators[i], nbOfAlgorithms);
   }
}

void
buildOutputResult2( Long * ranks, Collection & cold2, int nbOfAlgorithms ) {
   cold2.SETARRAY("Rank", Float, ranks, nbOfAlgorithms);
}

Errc
PRankSegmentationAlgorithms( Pobject ** colss,
			     int nbOfAlgorithms,
			     Collection & cold1, Collection & cold2,
			     int acceptable_error1, int priority1,
			     int acceptable_error2, int priority2,
			     int acceptable_error3, int priority3,
			     int acceptable_error4, int priority4,
			     int acceptable_error5, int priority5 ) {
   const std::string errorName = "Error";
   const Long nbOfIndicators = 5;

   EvaluationPerformance *performances = new EvaluationPerformance[nbOfAlgorithms];

   for (int i = 0; i < nbOfAlgorithms; i++) {
      Collection *cols = (Collection *)colss[i];
      EvaluationResult evaluationResult(nbOfIndicators * 2);

      ReadCollection(*cols,  errorName, nbOfIndicators, evaluationResult);
      performances[i] = ComputeIndicators(evaluationResult, acceptable_error1,
					  acceptable_error2, acceptable_error3,
					  acceptable_error4, acceptable_error5);
   }

   Long *ranks = new Long[nbOfAlgorithms];

   int priority[5];
   priority[0] = priority1;
   priority[1] = priority2;
   priority[2] = priority3;
   priority[3] = priority4;
   priority[4] = priority5;
   MultiIndicatorRanking(performances, ranks, nbOfAlgorithms, nbOfIndicators, priority);
   
   displayIndicators(performances, nbOfAlgorithms, nbOfIndicators);
   displayRanks(ranks, nbOfAlgorithms);

   buildOutputResult1(performances, cold1, nbOfAlgorithms, nbOfIndicators);
   buildOutputResult2(ranks, cold2, nbOfAlgorithms);

   return SUCCESS;
}

#ifdef MAIN

/*
 * Modify only the following constants, and the function call.
 */
#define	USAGE	"usage: %s [-v] acceptable_error1 priority1 acceptable_error2 priority2 acceptable_error3 priority3 acceptable_error4 priority4 acceptable_error5 priority5 indicator_collection* rank_collection"

#define	PARC	10
#define	FOUTC	2
#define	MASK	0

int
main( int argc, char *argv[] ) {
   Errc result;			// The result code of the execution.
   Pobject* mask;		// The region mask.
   Pobject** objin = 0;		// The input objects.
   Pobject** objs = 0;		// The source objects masked by the mask.
   Pobject* objout[FOUTC + 1];	// The input objects.
   Pobject* objd[FOUTC + 1];	// The source objects masked by the mask.
   char* parv[PARC + 1];	// The input parameters.

   if (!strcmp(argv[1], "-v")) {
      setVerbose();
      argc --;
      argv[1] = argv[0];
      argv++;
   }

   const int finc = argc - PARC - FOUTC - 1;

   if (finc > 1) {
      objin = new Pobject*[finc + 1];	// The output object.
      objs = new Pobject*[finc + 1];	// The result object of the execution.
   }
   
   ReadArgs(argc, argv, PARC, finc, FOUTC, &mask, objin, objs, objout, objd, parv, USAGE, MASK);

   objd[0] = new Collection;
   Collection* cold1 = (Collection*)objd[0];
   objd[1] = new Collection;
   Collection* cold2 = (Collection*)objd[1];
   
   for (int i = 0; i < finc; i++) {
      if (objs[i]->Type() != Po_Collection) {
   	 goto end;
      }
   }

   result = PRankSegmentationAlgorithms(objs, finc, 
					*cold1, *cold2, 
					atoi(parv[0]), atoi(parv[1]),
					atoi(parv[2]), atoi(parv[3]),
					atoi(parv[4]), atoi(parv[5]),
					atoi(parv[6]), atoi(parv[7]),
					atoi(parv[8]), atoi(parv[9]));
   
   if (result) {
      WriteArgs(argc, argv, PARC, finc, FOUTC, &mask, objin, objs, objout, objd, MASK);
   }
  end:
   if (finc > 0) {
      delete[] objs;
      delete[] objin;      
   }
   Exit(result);
   return 0;
}

#endif

